/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is  SessionPage.
 *
 * The Initial Developer of the Original Code is
 * Tairan Ma <matairan@users.sourceforge.net>
 *
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/**
 * SessionStoreManager
 * 
 * Overview
 * This object provide session manage service for the SessionPageController.
 */


/*
docShell capabilities. 
eg: browser.docShell["allow" + aCapability] = false;
*/
const CAPABILITIES = [
  "Subframes", "Plugins", "Javascript", "MetaRedirects", "Images"
];

var SessionStoreManager = {
  
  sessionList: [],
  sessionObj: null,
  restoreTabs:[],
  restoreDatas:[],

  init: function() 
  {    
    this.initSessionFileList();
    if (this.sessionList.length == 0) return;

    //set current session to the most recent.
    this.gotoSession(0);
   },
   
   initSessionFileList: function()
   {
    //init session files array.
    var sessionDir = this.getSessionPageDir();
    var fileEnum = sessionDir.directoryEntries;
    var aFile = null;
    while (fileEnum.hasMoreElements()) 
    {
      aFile  = fileEnum.getNext();
      aFile.QueryInterface(Components.interfaces.nsIFile);
      this.sessionList.push(aFile.lastModifiedTime);
    }
    this.sessionList.sort(function(a,b){return b-a});    
  	
   },
   
   removeSessionPageTabs: function()
   {
   	var winData = this.sessionObj.windows[0];
    if (!winData.tabs) 
    {
      winData.tabs = [];
    }

    // remove SessionPage Tab.
    for (var t = 0; t < winData.tabs.length; t++) 
    {
       var aTab = winData.tabs[t];
       if ((aTab.entries)&&(aTab.entries[aTab.index-1].url == "chrome://sessionpage/content/SessionPage.xul") )
           winData.tabs.splice(t,1);
    }
   	
 if (!winData._closedTabs) 
    {
      winData._closedTabs = [];
    }

    // remove SessionPage Tab.
    for (var t = 0; t < winData._closedTabs.length; t++) 
    {
       var aTab = winData._closedTabs[t].state;
       if ((aTab.entries)&&(aTab.entries[aTab.index-1].url == "chrome://sessionpage/content/SessionPage.xul") )
           winData._closedTabs.splice(t,1);
    }
   	
   },
   
   initCurrentSession: function()
   {
      var aSessionStoreFile = this.createSessionStoreFile();
      if (aSessionStoreFile.exists())	
      {
        this.sessionObj = this.convertToObject(this.readSessionFile(aSessionStoreFile));
        
        this.removeSessionPageTabs();
            }   	
   },
   
   
  gotoSession: function(sessionIndex)
  {
    var aSessionFile = this.getSessionPageDir();
    aSessionFile.append("sessionpage_"+this.sessionList[sessionIndex]+".js");
    this.sessionObj = this.convertToObject(this.readSessionFile(aSessionFile));
    var winData = this.sessionObj.windows[0];
    if (!winData.tabs) 
    {
      winData.tabs = [];
    }
    this.removeSessionPageTabs();
    // remove SessionPage Tab.
    /*for (var t = 0; t < winData.tabs.length; t++) 
    {
      var aTab = winData.tabs[t];
      if ((aTab.entries)&&(aTab.entries[aTab.index-1].url == "chrome://sessionpage/content/SessionPage.xul") )
        winData.tabs.splice(t,1);
    }*/
  }, 
   
  isAlreadyOpened: function(aTabBrowser,aTabData)
  {
  	if ((!aTabData)||(aTabData.entries.length == 0)) return {opened:false,index:-1};  	
    for (var t = 0; t < aTabBrowser.browsers.length; t++) 
    {
    	if (aTabBrowser.browsers[t].currentURI.spec == aTabData.entries[aTabData.index-1].url)
	    {
        return {opened:true,index:t};
      }
    } 
    return {opened:false,index:-1};  	
  }, 
   
  openPage: function(aPageIdx, isClosedTab)
  {
   	if (this.sessionObj == null) return;
    var windowMediator = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);  
    var aWindow = windowMediator.getMostRecentWindow("navigator:browser");
    var tabbrowser = aWindow.getBrowser();
   	var aTabData = isClosedTab ? this.sessionObj.windows[0]._closedTabs[aPageIdx].state : this.sessionObj.windows[0].tabs[aPageIdx];
    var isOpened = this.isAlreadyOpened(tabbrowser,aTabData);
    if (isOpened.opened)
    {
	     //tabbrowser.tabContainer.selectedIndex = isOpened.index;
    }
     else
    {
    	 var _this = this;
 	     aWindow.setTimeout( function(){_this.restoreOneTab(aWindow,aTabData);},0);   	 	
    }
  },

  openAll: function(isClosedTabs)
  {
   	if (this.sessionObj == null) return;
    if (!this.sessionObj.windows[0]) 
    {
        return; // nothing to restore
    }
    var windowMediator = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);  
    var aWindow = windowMediator.getMostRecentWindow("navigator:browser");
    var winData = this.sessionObj.windows[0];               
    var newTabCount = isClosedTabs ?  winData._closedTabs.length :winData.tabs.length;
    var _this = this;
    if (winData.cookies) 
    {
      this.restoreCookies(winData.cookies);
    }
    if (isClosedTabs)
      {for (var t = 0; t < newTabCount; t++) 
        //aWindow.setTimeout( function(){_this.restoreOneTab(aWindow,winData._closedTabs[t].state);},0);      	
        this.restoreOneTab(aWindow,winData._closedTabs[t].state);        
      }
    else
      {for (var t = 0; t < newTabCount; t++)       
        //aWindow.setTimeout( function(){_this.restoreOneTab(aWindow,winData.tabs[t]);},0);    
	    this.restoreOneTab(aWindow,winData.tabs[t]);
      }

  },
  
  
  restoreOneTab: function(aWindow,aTabData)
  {
  	 if (!aTabData) return;
    var isOpened = this.isAlreadyOpened(aWindow.getBrowser(),aTabData);
    if (isOpened.opened) return;

    // helper hash for ensuring unique frame IDs
    var idMap = { used: {} };
    
    var tab = aWindow.getBrowser().addTab();
    var browser = aWindow.getBrowser().getBrowserForTab(tab);
    var history = browser.webNavigation.sessionHistory;
    
    if (history.count > 0) {
      history.PurgeHistory(history.count);
    }
    history.QueryInterface(Components.interfaces.nsISHistoryInternal);
    
    if (!aTabData.entries) {
      aTabData.entries = [];
    }
    
    browser.markupDocumentViewer.textZoom = parseFloat(aTabData.zoom || 1);
    
    for (var i = 0; i < aTabData.entries.length; i++) {
      history.addEntry(this.deserializeHistoryEntry(aTabData.entries[i], idMap), true);
    }
    
    // make sure to reset the capabilities and attributes, in case this tab gets reused
    var disallow = (aTabData.disallow)?aTabData.disallow.split(","):[];
    CAPABILITIES.forEach(function(aCapability) {
      browser.docShell["allow" + aCapability] = disallow.indexOf(aCapability) == -1;
    });
    
    // restore those aspects of the currently active documents
    // which are not preserved in the plain history entries
    // (mainly scroll state and text data)
//    this.restoreTabs.push(browser);
//    this.restoreDatas.push({
//                            Data: aTabData.entries[activeIndex] || {},
//                            Text: aTabData.text || "",
 //                           Tab: tab
  //                         });
//    var _this = this;
//    browser.addEventListener("load",
//                             function(evt)
//                              {
//                              	 _this.restoreDocumentText(evt);
//                              } , true);                           
    
    
    var activeIndex = (aTabData.index || aTabData.entries.length) - 1;
    try {
      browser.webNavigation.gotoIndex(activeIndex);
    }
    catch (ex) { } // ignore an invalid aTabData.index

    browser.__SP_restore_data = aTabData.entries[activeIndex] || {};
    browser.__SP_restore_text = aTabData.text || "";
    browser.__SP_restore_tab = tab;
    browser.__SP_restore = this.restoreDocument_proxy;
    browser.addEventListener("load", browser.__SP_restore, true);
    
  },
 
  /**
   * expands serialized history data into a session-history-entry instance
   * @param aEntry
   *        Object containing serialized history data for a URL
   * @param aIdMap
   *        Hash for ensuring unique frame IDs
   * @returns nsISHEntry
   */
  deserializeHistoryEntry: function(aEntry, aIdMap) {
    var shEntry = Components.classes["@mozilla.org/browser/session-history-entry;1"].
                  createInstance(Components.interfaces.nsISHEntry);
    
    var ioService = Components.classes["@mozilla.org/network/io-service;1"].
                    getService(Components.interfaces.nsIIOService);
    shEntry.setURI(ioService.newURI(aEntry.url, null, null));
    shEntry.setTitle(aEntry.title || aEntry.url);
    shEntry.setIsSubFrame(aEntry.subframe || false);
    shEntry.loadType = Components.interfaces.nsIDocShellLoadInfo.loadHistory;
    
    if (aEntry.cacheKey) {
      var cacheKey = Components.classes["@mozilla.org/supports-PRUint32;1"].
                     createInstance(Components.interfaces.nsISupportsPRUint32);
      cacheKey.data = aEntry.cacheKey;
      shEntry.cacheKey = cacheKey;
    }
    if (aEntry.ID) {
      // get a new unique ID for this frame (since the one from the last
      // start might already be in use)
      var id = aIdMap[aEntry.ID] || 0;
      if (!id) {
        for (id = Date.now(); aIdMap.used[id]; id++);
        aIdMap[aEntry.ID] = id;
        aIdMap.used[id] = true;
      }
      shEntry.ID = id;
    }
    
    var scrollPos = (aEntry.scroll || "0,0").split(",");
    scrollPos = [parseInt(scrollPos[0]) || 0, parseInt(scrollPos[1]) || 0];
    shEntry.setScrollPosition(scrollPos[0], scrollPos[1]);
    
    if (aEntry.postdata) {
      var stream = Components.classes["@mozilla.org/io/string-input-stream;1"].
                   createInstance(Components.interfaces.nsIStringInputStream);
      stream.setData(aEntry.postdata, -1);
      shEntry.postData = stream;
    }
    
    if (aEntry.children && shEntry instanceof Components.interfaces.nsISHContainer) {
      for (var i = 0; i < aEntry.children.length; i++) {
        shEntry.AddChild(this.deserializeHistoryEntry(aEntry.children[i], aIdMap), i);
      }
    }
    
    return shEntry;
  },
  
  /**
   * Restores cookies to cookie service
   * @param aCookies
   *        Array of cookie data
   */ 
  restoreCookies: function(aCookies) 
  {
    var cookieService = Components.classes["@mozilla.org/cookieService;1"].
                        getService(Components.interfaces.nsICookieService);
    var ioService = Components.classes["@mozilla.org/network/io-service;1"].
                    getService(Components.interfaces.nsIIOService);
    
    for (var i = 1; i <= aCookies.count; i++) {
      try {
        cookieService.setCookieString(ioService.newURI(aCookies["domain" + i], null, null), null, aCookies["value" + i] + "expires=0;", null);
      }
      catch (ex) { debug(ex); } // don't let a single cookie stop recovering (might happen if a user tried to edit the session file)
    }
  },
  
  copySessionFile: function(aSessionStoreFile)
  {
    if (aSessionStoreFile.exists())	
    {
      //remove oldest sessionpage file.
      
      var keepSessions = getPref("extensions.sessionpage.keepsessions",10);
      if (keepSessions <1) keepSessions = 1;
      if (this.sessionList.length == 0) this.initSessionFileList();
      while (this.sessionList.length >=  keepSessions)
      {
        var aSessionPageFile = this.getSessionPageDir();
        aSessionPageFile.append("sessionpage_"+this.sessionList.pop()+".js")
        if (aSessionPageFile.exists()) aSessionPageFile.remove(false);        
      } 
      
      //do copy
      var aSessionPageDir = this.getSessionPageDir();  
	  aSessionStoreFile.copyTo(aSessionPageDir,"sessionpage_"+aSessionStoreFile.lastModifiedTime+".js");
	}
  }, 
 
   /**
   * Restore properties to a loaded document
   */
  restoreDocument_proxy: function sss_restoreDocument_proxy(aEvent) {
    // wait for the top frame to be loaded completely
    if (!aEvent || !aEvent.originalTarget || !aEvent.originalTarget.defaultView || aEvent.originalTarget.defaultView != aEvent.originalTarget.defaultView.top) {
      return;
    }
    
    var textArray = this.__SP_restore_text ? this.__SP_restore_text.split(" ") : [];
    function restoreTextData(aContent, aPrefix) {
      textArray.forEach(function(aEntry) {
        if (/^((?:\d+\|)*)(#?)([^\s=]+)=(.*)$/.test(aEntry) && (!RegExp.$1 || RegExp.$1 == aPrefix)) {
          var document = aContent.document;
          var node = RegExp.$2 ? document.getElementById(RegExp.$3) : document.getElementsByName(RegExp.$3)[0] || null;
          if (node && "value" in node) {
            node.value = decodeURI(RegExp.$4);
            
            var event = document.createEvent("UIEvents");
            event.initUIEvent("input", true, true, aContent, 0);
            node.dispatchEvent(event);
          }
        }
      });
    }
    
    function restoreTextDataAndScrolling(aContent, aData, aPrefix) {
      restoreTextData(aContent, aPrefix);
      if (aData.innerHTML) {
        aContent.setTimeout(function(aHTML) { if (this.document.designMode == "on") { this.document.body.innerHTML = aHTML; } }, 0, aData.innerHTML);
      }
      if (aData.scroll && /(\d+),(\d+)/.test(aData.scroll)) {
        aContent.scrollTo(RegExp.$1, RegExp.$2);
      }
      for (var i = 0; i < aContent.frames.length; i++) {
        if (aData.children && aData.children[i]) {
          restoreTextDataAndScrolling(aContent.frames[i], aData.children[i], i + "|" + aPrefix);
        }
      }
    }
    
    var content = XPCNativeWrapper(aEvent.originalTarget).defaultView;
    if (this.currentURI.spec == "about:config") {
      // unwrap the document for about:config because otherwise the properties
      // of the XBL bindings - as the textbox - aren't accessible (see bug 350718)
      content = content.wrappedJSObject;
    }
    restoreTextDataAndScrolling(content, this.__SP_restore_data, "");
    
    // notify the tabbrowser that this document has been completely restored
    var event = this.ownerDocument.createEvent("Events");
    event.initEvent("SSTabRestored", true, false);
    this.__SP_restore_tab.dispatchEvent(event);
    
    this.removeEventListener("load", this.__SP_restore, true);
    delete this.__SP_restore_data;
    delete this.__SP_restore_text;
    delete this.__SP_restore_tab;
    delete this.__SP_restore;
  },
   
  savePages: function()
  {
    var aSessionStoreFile = this.createSessionStoreFile();
    this.copySessionFile(aSessionStoreFile);
  },
   
   
  readSessionFile: function(aFile) 
  {
    try 
    {
      var stream = Components.classes["@mozilla.org/network/file-input-stream;1"].
                   createInstance(Components.interfaces.nsIFileInputStream);
      stream.init(aFile, 0x01, 0, 0);
      var cvstream = Components.classes["@mozilla.org/intl/converter-input-stream;1"].
                     createInstance(Components.interfaces.nsIConverterInputStream);
      cvstream.init(stream, "UTF-8", 1024, Components.interfaces.nsIConverterInputStream.DEFAULT_REPLACEMENT_CHARACTER);
      
      var content = "";
      var data = {};
      while (cvstream.readString(4096, data)) 
      {
        content += data.value;
      }
      cvstream.close();
      return content.replace(/\r\n?/g, "\n");
    }
    catch (ex) { alert("SessionPage Read File Error!")} // inexisting file?
    
    return null;
  },
  
  convertToObject: function(aStr)
  {
    var s = new Components.utils.Sandbox("about:blank");
    return Components.utils.evalInSandbox(aStr, s);
  },
  
  getSessionPageDir: function()
  {
    var aDirService = Components.classes['@mozilla.org/file/directory_service;1'].getService(Components.interfaces.nsIProperties);
	var aDir = aDirService.get("ProfD", Components.interfaces.nsIFile);
	aDir.append("SessionPage");
	if ( !aDir.exists() )
      aDir.create(aDir.DIRECTORY_TYPE, 0700);
    return aDir;
  },
  
  createSessionStoreFile: function()
  {
    var SSDisabled = false;
    if (!getPref("browser.sessionstore.enabled", true))
    {
      SSDisabled = true;
      setPref("browser.sessionstore.enabled", true);
    }
    var ss = Components.classes["@mozilla.org/browser/sessionstore;1"].getService(Components.interfaces.nsISessionStore);
    ss.saveState;
    
    //get the file
    var aDirService = Components.classes['@mozilla.org/file/directory_service;1'].getService(Components.interfaces.nsIProperties);
    var aSessionStoreFile = aDirService.get("ProfD", Components.interfaces.nsILocalFile);
    aSessionStoreFile.append("sessionstore.js");
    //restore the SS pref
	if (SSDisabled)  setPref("browser.sessionstore.enabled", false);
	
    return aSessionStoreFile; 
  
  }
}