/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is  SessionPage.
 *
 * The Initial Developer of the Original Code is
 * Tairan Ma <matairan@users.sourceforge.net>
 *
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/**
 * SessionPageController
 * 
 * Overview
 * This object controls the UI interactivities.
 */

var SessionPageController = {
  currentSessionState : "unclosed",

  init: function()
	{
//		
//    if (SessionStoreManager.sessionObj == null) 
//    {
    SessionStoreManager.init();
//    }  
    this.initSessionList();
    this.initPageList(false);
  },
  
  initSessionList: function()
  {
    var aMenuList = document.getElementById("sessionList");
    var aSessionCount = SessionStoreManager.sessionList.length;
    if (aSessionCount==0) return;
    for( var i = 0; i < aSessionCount; i++)
    {
      var aDate = new Date(SessionStoreManager.sessionList[i]); 
      var aMenu = aMenuList.appendItem(aDate.toLocaleString(),i);
      aMenu.setAttribute("oncommand", "SessionPageController.changeSession("+i+");");
    }
    
    //I just find this way to make the menulist redrow itself.
    //It's important,or the buttons on the page won't work at first click.
    aMenuList.selectedIndex = 0;

  },

  initPageList: function(isClosedTabs)
  {
    if (SessionStoreManager.sessionObj == null) return;
    var aTabs = isClosedTabs ? SessionStoreManager.sessionObj.windows[0]._closedTabs : SessionStoreManager.sessionObj.windows[0].tabs;	  	  
    var aTabCount = aTabs.length;
    var aBox = document.getElementById("sessionpage_box");
    while (aBox.hasChildNodes()) aBox.removeChild(aBox.lastChild); //clear list.
    
    for (var t = 0; t < aTabCount; t++) 
    {
      var aEntry = isClosedTabs ? aTabs[t].state.entries[aTabs[t].state.index-1] : aTabs[t].entries[aTabs[t].index-1];
      var aItem = document.createElement("checkbox");
      aItem.setAttribute("id","session_page_item"+t);
      aItem.setAttribute("label",aEntry.title);
      aItem.setAttribute("checked",true);
      aItem.setAttribute("tooltiptext",aEntry.url);
	  // Hacked by Chappell.Wat from QuChao.com @ 2008-7-13 start
      aItem.setAttribute('num', t);
      aItem.addEventListener('click', function(evt) {
		  if (2 === evt.button) {
			  var aItem = evt.target;
			  SessionPageController.linkClicked(aItem.id);
			  SessionPageController.openPage(aItem.getAttribute('num'));
		  }
      }, false);
	  // Hacked by Chappell.Wat from QuChao.com @ 2008-7-13 end
      aBox.appendChild(aItem);
    }		
  },


  linkClicked: function(aLinkID)
  {
    var aLabel = document.getElementById(aLinkID);
    aLabel.setAttribute("class","visited_url");  
  },

  resetSessionState: function()
  {
    var aRadio = document.getElementById("radio_unclosed");
    aRadio.setAttribute("selected",true);
    aRadio = document.getElementById("radio_closed");
    aRadio.setAttribute("selected",false);
    this.currentSessionState = "unclosed";
  },
    
  changeSession: function(aSessionIndex)
  {
    this.resetSessionState();
    SessionStoreManager.gotoSession(aSessionIndex);
    this.initPageList(false);
  },
  
  showCurrentSession: function()
  {
    this.resetSessionState();	
    SessionStoreManager.initCurrentSession();
    this.initPageList(false);
    // String service
    var stringService = Components.classes["@mozilla.org/intl/stringbundle;1"]
															.getService(Components.interfaces.nsIStringBundleService);
    var strbundle = stringService.createBundle("chrome://sessionpage/locale/sessionpage.properties");
	//strbundle.GetStringFromName(name);
    var currentSessionLabel=strbundle.GetStringFromName("CurrentSession");
  	var sessionMenuList = document.getElementById("sessionList");
  	sessionMenuList.selectedIndex =-1;
  	sessionMenuList.setAttribute("label",currentSessionLabel);
  },

 //Open all session pages
 
  OpenSelected: function()
  {
    var windowMediator = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);  
    var gb = windowMediator.getMostRecentWindow("navigator:browser").getBrowser();
    var currentTab = gb.mCurrentTab;
    var checkClose = document.getElementById("check_closeAfterOpen");
    var aBox = document.getElementById("sessionpage_box");
    if (aBox.hasChildNodes()) 
    {
      var aItem = null;
      for ( var i = 0; i<aBox.childNodes.length; i++)
      {
        aItem = aBox.childNodes[i];
        if (aItem.getAttribute("checked") == "true") {
            this.currentSessionState == "unclosed" ?
   	        SessionStoreManager.openPage(i,false)
           :
   	        SessionStoreManager.openPage(i,true);
            // Hacked by Chappell.Wat from QuChao.com @ 2008-7-13 start
            SessionPageController.linkClicked(aItem.id);
            // Hacked by Chappell.Wat from QuChao.com @ 2008-7-13 end
        }
      }
    }
    if (checkClose.getAttribute("checked") == "true") {
       setTimeout(function(){gb.removeTab(currentTab)}, 0);
    }    
  },
  
  checkAll: function(isCheck)
  {
    var aBox = document.getElementById("sessionpage_box");
    if (aBox.hasChildNodes()) 
    {
      var aItem = null;
      for ( var i = 0; i<aBox.childNodes.length; i++)
      {
        aItem = aBox.childNodes[i];
        aItem.setAttribute("checked",isCheck);
      }
    }
  },
  
  toggleCheck: function()
  {
    var aBox = document.getElementById("sessionpage_box");
    if (aBox.hasChildNodes()) 
    {
      var aItem = null;
      for ( var i = 0; i<aBox.childNodes.length; i++)
      {
        aItem = aBox.childNodes[i];
        aItem.getAttribute("checked") == "true" ?
   	     aItem.setAttribute("checked",false)
        :
   	     aItem.setAttribute("checked",true);
      }
    }
  },
 
  openPage:function(aPageIndex)
  {
     this.currentSessionState == "unclosed" ?
   	   SessionStoreManager.openPage(aPageIndex,false)
     :
   	   SessionStoreManager.openPage(aPageIndex,true);
  },
  
  openAbout: function()
  {
  	var aWin; 
    window.opener != null ? aWin = window.opener : aWin = window;
    aWin.open("chrome://sessionpage/content/about.xul","About SessionPage","chrome,modal,centerscreen");
  },

  openOptions: function()
  {
  	var aWin; 
    window.opener != null ? aWin = window.opener : aWin = window;
    aWin.open("chrome://sessionpage/content/options.xul","SessionPage Options","chrome,modal,centerscreen");
  },
  
  changeSessionTabs: function()
  {
    var aRadio = document.getElementById("radio_unclosed");
    var tmpState = ( aRadio.selected ? "unclosed" : "closed");
    if (tmpState == this.currentSessionState) return;
    this.currentSessionState = tmpState;
    tmpState == "unclosed" ? this.initPageList(false) : this.initPageList(true);
    
  }

};



